from pyramid.security import Allow
from pyramid.security import Everyone

import model as m

class AssertionApp(object):
    def __init__(self, badge):
        self.badge = badge

    def __getitem__(self, key):
        return m.Assertion.query.filter_by(
            recipient=key,
            badge=self.badge,
        ).one()

class RootApp(object):
    __name__ = None
    __parent__ = None

    def __getitem__(self, key):
        if key == 'assertions':
            return self

        badge = m.Badge.query.filter_by(id=key).one()
        return AssertionApp(badge=badge)

def get_root(request):
    return RootApp()




import transaction

import tw2.core as twc

from mako.template import Template as t
from pyramid.view import (
    view_config,
    forbidden_view_config,
)

from pyramid.httpexceptions import HTTPFound

from pyramid.security import (
    authenticated_userid,
    remember,
    forget,
)

import model as m
import widgets


# TODO -- really wield tw2.sqla here
@view_config(route_name='admin', renderer='admin.mak')
def admin(request):
    logged_in = authenticated_userid(request)

    if logged_in != request.registry.settings['tahrir.admin']:
        return HTTPFound(location='/')

    is_awarded = lambda a: logged_in and a.person.email == logged_in
    awarded_assertions = filter(is_awarded, m.Assertion.query.all())


    name_lookup = {
        'issuerform': widgets.IssuerForm,
        'badgeform': widgets.BadgeForm,
        'assertionform': widgets.AssertionForm,
        'personform': widgets.PersonForm,
    }

    for key in name_lookup:
        if any([k.startswith(key) for k in request.params]):
            w = name_lookup[key]

            try:
                data = w.validate(request.params)
                w.validated_request(request, data,
                                    protect_prm_tamp=False)
                return HTTPFound(location='/admin')
            except twc.ValidationError as e:
                print e.widget

    return dict(
        is_admin=True,
        logged_in=logged_in,
        awarded_assertions=awarded_assertions,
        base_url=request.registry.settings['tahrir.base_url'],
        title=request.registry.settings['tahrir.title'],
        issuer_form = widgets.IssuerForm,
        badge_form = widgets.BadgeForm,
        assertion_form = widgets.AssertionForm,
        person_form = widgets.PersonForm,
    )


@view_config(route_name='home', renderer='index.mak')
def index(request):
    logged_in = authenticated_userid(request)
    is_admin = logged_in == request.registry.settings['tahrir.admin']
    is_awarded = lambda a: logged_in and a.person.email == logged_in
    awarded_assertions = filter(is_awarded, m.Assertion.query.all())
    return dict(
        is_admin=is_admin,
        issuers=m.Issuer.query.all(),
        awarded_assertions=awarded_assertions,
        base_url=request.registry.settings['tahrir.base_url'],
        logged_in=logged_in,
        title=request.registry.settings['tahrir.title'],
    )


@view_config(context=m.Assertion, renderer='json')
def json(context, request):
    return context.__json__()


@view_config(route_name='login', renderer='login.mak')
@forbidden_view_config(renderer='login.mak')
def login(request):
    login_url = request.resource_url(request.context, 'login')
    referrer = request.url
    if referrer == login_url:
        referrer = '/' # never use the login form itself as came_from
    came_from = request.params.get('came_from', referrer)
    message = ''
    email = ''
    if 'form.submitted' in request.params:
        email = request.params['email']
        if m.Person.query.filter_by(email=email).count() == 0:
            new_user = m.Person(email=email)
            m.DBSession.add(new_user)

        # NOTE -- there is no way to fail login here :D
        # TODO -- validate the email address
        if True:
            headers = remember(request, email)
            return HTTPFound(location = came_from,
                             headers = headers)
        message = 'Failed login'

    return dict(
        title=request.registry.settings['tahrir.title'],
        message = message,
        url = request.application_url + '/login',
        came_from = came_from,
        email = email,
        )


@view_config(route_name='logout')
def logout(request):
    headers = forget(request)
    return HTTPFound(location = request.resource_url(request.context),
                     headers = headers)




import os

from pyramid.config import Configurator
from sqlalchemy import engine_from_config

from pyramid.authentication import AuthTktAuthenticationPolicy
from pyramid.authorization import ACLAuthorizationPolicy

from .app import get_root
from .model import DBSession
from .widgets import SavingFileField

def main(global_config, **settings):
    """ This function returns a Pyramid WSGI application.
    """

    required_keys = [
        'tahrir.salt',
        'tahrir.pngs.uri',
        'tahrir.admin',
        'tahrir.title',
        'tahrir.base_url',
    ]

    # validate the config
    for key in required_keys:
        if key not in settings:
            raise ValueError("%s required in settings." % key)

    # Make data dir if it doesn't already exist.
    settings['tahrir.pngs.uri'] = os.path.abspath(settings['tahrir.pngs.uri'])
    if not os.path.exists(settings['tahrir.pngs.uri']):
        os.makedirs(settings['tahrir.pngs.uri'])

    # Set that directory on the filefield widget.
    SavingFileField.png_dir = settings['tahrir.pngs.uri']

    # start setting things up
    engine = engine_from_config(settings, 'sqlalchemy.')
    DBSession.configure(bind=engine)

    authn_policy = AuthTktAuthenticationPolicy(
        secret='verysecret',
    )
    authz_policy = ACLAuthorizationPolicy()

    config = Configurator(settings=settings, root_factory=get_root)

    config.set_authentication_policy(authn_policy)
    config.set_authorization_policy(authz_policy)

    config.add_static_view(
        'static', 'static', cache_max_age=3600)
    config.add_static_view(
        'pngs', settings['tahrir.pngs.uri'], cache_max_age=3600)
    config.add_route('home', '/')
    config.add_route('admin', '/admin')
    config.add_route('login', '/login')
    config.add_route('logout', '/logout')

    config.scan()

    return config.make_wsgi_app()





import model as m
import tw2.core as twc
import tw2.sqla as tws
import tw2.forms as twf

import hashlib
import os

import logging
log = logging.getLogger(__name__)


class IssuerForm(tws.DbFormPage):
    entity = m.Issuer
    class child(twf.TableForm):
        id = twf.HiddenField()
        origin = twf.TextField(validator=twc.Required)
        name = twf.TextField(validator=twc.Required)
        org = twf.TextField(validator=twc.Required)
        contact = twf.TextField(validator=twc.Required)


class SavingFileField(twf.FileField):
    validator = twf.FileValidator(extesion='.png')

    png_dir = None

    def _save_file(self, filename, value):
        if not self.png_dir:
            raise twc.ValidationError("Image saving is misconfigured.")

        filename = os.path.sep.join([self.png_dir, filename])
        log.info("Writing image to %s" % filename)

        with open(filename, 'wb') as f:
            f.write(value)

        return True

    def _validate(self, value, state=None):
        try:
            super(SavingFileField, self)._validate(value, state)
            filename = hashlib.md5(self.value.filename).hexdigest() + ".png"
            self._save_file(filename, self.value.value)
            self.value = filename
            return self.value
        except twc.ValidationError:
            sys.exit(1)
            self.value = None
            raise


class BadgeForm(tws.DbFormPage):
    entity = m.Badge
    class child(twf.TableForm):
        id = twf.HiddenField()
        name = twf.TextField(validator=twc.Required)
        image = SavingFileField
        description = twf.TextArea(
            rows=5,
            cols=25,
            validator=twc.Required,
        )
        criteria = twf.TextField(validator=twc.Required)
        issuer = tws.DbRadioButtonList(entity=m.Issuer)


class AssertionForm(tws.DbFormPage):
    entity = m.Assertion
    class child(twf.TableForm):
        id = twf.HiddenField()
        badge = tws.DbRadioButtonList(entity=m.Badge)
        person = tws.DbRadioButtonList(entity=m.Person)


class PersonForm(tws.DbFormPage):
    entity = m.Person
    class child(twf.TableForm):
        id = twf.HiddenField()
        email = twf.TextField(validator=twc.Required)



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()

requires = [
    'pyramid',
    'SQLAlchemy',
    'transaction',
    'pyramid_tm',
    'zope.sqlalchemy',
    "weberror",
    "tw2.sqla",
    "formencode",  # For file uploads
    ]

setup(name='tahrir',
      version='0.1.2',
      description='A pyramid app for issuing your own Open Badges',
      long_description=README,
      classifiers=[
        "Programming Language :: Python",
        "Framework :: Pyramid",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        ],
      author='Ralph Bean',
      author_email='rbean@redhat.com',
      url='http://github.com/ralphbean/tahrir',
      keywords='web wsgi bfg pylons pyramid',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      test_suite='tahrir',
      install_requires=requires,
      entry_points="""\
      [paste.app_factory]
      main = tahrir:main
      [console_scripts]
      initialize_tahrir_db = tahrir.scripts.initializedb:main
      """,
      )

